@@ -28,6 +28,7 @@ import org.apache.maven.wagon.Wagon;
 import org.apache.maven.wagon.authentication.AuthenticationInfo;
 import org.apache.maven.wagon.authorization.AuthorizationException;
 import org.apache.maven.wagon.proxy.ProxyInfo;
+import org.apache.maven.wagon.proxy.ProxyInfoProvider;
 import org.apache.maven.wagon.repository.Repository;
 import org.apache.maven.wagon.resource.Resource;
 import org.codehaus.plexus.util.FileUtils;
@@ -76,6 +77,8 @@ import java.util.zip.GZIPOutputStream;
 public abstract class HttpWagonTestCase
     extends StreamingWagonTestCase
 {
+    public static final int SC_TOO_MANY_REQUESTS = 429;
+
     private Server server;
 
     protected void setupWagonTestingFixtures()
@@ -204,6 +207,56 @@ public abstract class HttpWagonTestCase
         assertEquals( "Maven-Wagon/1.0", handler.headers.get( "User-Agent" ) );
     }
 
+    public void testUserAgentHeaderIsPresentByDefault()
+        throws Exception
+    {
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+        Server server = new Server( 0 );
+        TestHeaderHandler handler = new TestHeaderHandler();
+        server.setHandler( handler );
+        addConnectors( server );
+        server.start();
+        wagon.connect( new Repository( "id", getProtocol() + "://localhost:" 
+          + server.getConnectors()[0].getLocalPort() ) );
+        wagon.getToStream( "resource", new ByteArrayOutputStream() );
+        wagon.disconnect();
+        server.stop();
+
+        assertNotNull( "default User-Agent header of wagon provider should be present",
+                       handler.headers.get( "User-Agent" ) );
+    }
+
+    public void testUserAgentHeaderIsPresentOnlyOnceIfSetMultipleTimes()
+        throws Exception
+    {
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+
+        // 1. set User-Agent header via HttpConfiguration
+        Properties headers1 = new Properties();
+        headers1.setProperty( "User-Agent", "test-user-agent" );
+        setHttpHeaders( wagon, headers1 );
+
+        // 2. redundantly set User-Agent header via setHttpHeaders()
+        Properties headers2 = new Properties();
+        headers2.setProperty( "User-Agent", "test-user-agent" );
+        Method setHttpHeaders = wagon.getClass().getMethod( "setHttpHeaders", Properties.class );
+        setHttpHeaders.invoke( wagon, headers2 );
+
+        Server server = new Server( 0 );
+        TestHeaderHandler handler = new TestHeaderHandler();
+        server.setHandler( handler );
+        addConnectors( server );
+        server.start();
+        wagon.connect( new Repository( "id", getProtocol() + "://localhost:"
+          + server.getConnectors()[0].getLocalPort() ) );
+        wagon.getToStream( "resource", new ByteArrayOutputStream() );
+        wagon.disconnect();
+        server.stop();
+
+        assertEquals( "test-user-agent", handler.headers.get( "User-Agent" ) );
+
+    }
+
     protected abstract void setHttpHeaders( StreamingWagon wagon, Properties properties );
 
     protected void addConnectors( Server server )
@@ -261,13 +314,13 @@ public abstract class HttpWagonTestCase
                 {
                     if ( called.get() )
                     {
-                        response.setStatus( 200 );
+                        response.setStatus( HttpServletResponse.SC_OK );
                         ( (Request) request ).setHandled( true );
                     }
                     else
                     {
                         called.set( true );
-                        response.setStatus( 429 );
+                        response.setStatus( SC_TOO_MANY_REQUESTS );
                         ( (Request) request ).setHandled( true );
 
                     }
@@ -414,7 +467,7 @@ public abstract class HttpWagonTestCase
                     else
                     {
                         called.set( true );
-                        response.setStatus( 429 );
+                        response.setStatus( SC_TOO_MANY_REQUESTS );
                         ( (Request) request ).setHandled( true );
                     }
                 }
@@ -554,12 +607,45 @@ public abstract class HttpWagonTestCase
 
         if ( supportProxyPreemptiveAuthentication() )
         {
-            assertEquals( 200, handler.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( HttpServletResponse.SC_OK, handler.handlerRequestResponses.get( 0 ).responseCode );
         }
         else
         {
-            assertEquals( 407, handler.handlerRequestResponses.get( 0 ).responseCode );
-            assertEquals( 200, handler.handlerRequestResponses.get( 1 ).responseCode );
+            assertEquals( HttpServletResponse.SC_PROXY_AUTHENTICATION_REQUIRED,
+                          handler.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( HttpServletResponse.SC_OK, handler.handlerRequestResponses.get( 1 ).responseCode );
+        }
+
+    }
+
+    public void testProxiedRequestWithAuthenticationWithProvider()
+        throws Exception
+    {
+        final ProxyInfo proxyInfo = createProxyInfo();
+        proxyInfo.setUserName( "user" );
+        proxyInfo.setPassword( "secret" );
+        AuthorizingProxyHandler handler = new AuthorizingProxyHandler();
+
+        ProxyInfoProvider proxyInfoProvider = new ProxyInfoProvider()
+        {
+            public ProxyInfo getProxyInfo( String protocol )
+            {
+                return proxyInfo;
+            }
+        };
+        runTestProxiedRequestWithProvider( proxyInfoProvider, handler );
+
+        assertTrue( handler.headers.containsKey( "Proxy-Authorization" ) );
+
+        if ( supportProxyPreemptiveAuthentication() )
+        {
+            assertEquals( HttpServletResponse.SC_OK, handler.handlerRequestResponses.get( 0 ).responseCode );
+        }
+        else
+        {
+            assertEquals( HttpServletResponse.SC_PROXY_AUTHENTICATION_REQUIRED,
+                          handler.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( HttpServletResponse.SC_OK, handler.handlerRequestResponses.get( 1 ).responseCode );
         }
 
     }
@@ -595,7 +681,8 @@ public abstract class HttpWagonTestCase
 
         String redirectUrl = protocol + "://localhost:" + server.getConnectors()[0].getLocalPort();
 
-        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, null );
+        RedirectHandler redirectHandler =
+            new RedirectHandler( "Found", HttpServletResponse.SC_SEE_OTHER, redirectUrl, null );
 
         redirectServer.setHandler( redirectHandler );
 
@@ -615,10 +702,8 @@ public abstract class HttpWagonTestCase
             String found = FileUtils.fileRead( tmpResult );
             assertEquals( "found:'" + found + "'", "Hello, World!", found );
 
-            assertEquals( 1, handler.handlerRequestResponses.size() );
-            assertEquals( 200, handler.handlerRequestResponses.get( 0 ).responseCode );
-            assertEquals( 1, redirectHandler.handlerRequestResponses.size() );
-            assertEquals( 302, redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
+            checkHandlerResult( handler.handlerRequestResponses, HttpServletResponse.SC_OK );
+            checkHandlerResult( redirectHandler.handlerRequestResponses, HttpServletResponse.SC_FOUND );
         }
         finally
         {
@@ -661,7 +746,8 @@ public abstract class HttpWagonTestCase
 
         String redirectUrl = protocol + "://localhost:" + server.getConnectors()[0].getLocalPort();
 
-        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, null );
+        RedirectHandler redirectHandler =
+            new RedirectHandler( "Found", HttpServletResponse.SC_SEE_OTHER, redirectUrl, null );
 
         redirectServer.setHandler( redirectHandler );
 
@@ -677,10 +763,8 @@ public abstract class HttpWagonTestCase
             String found = FileUtils.fileRead( tmpResult );
             assertEquals( "found:'" + found + "'", "Hello, World!", found );
 
-            assertEquals( 1, handler.handlerRequestResponses.size() );
-            assertEquals( 200, handler.handlerRequestResponses.get( 0 ).responseCode );
-            assertEquals( 1, redirectHandler.handlerRequestResponses.size() );
-            assertEquals( 302, redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
+            checkHandlerResult( handler.handlerRequestResponses, HttpServletResponse.SC_OK );
+            checkHandlerResult( redirectHandler.handlerRequestResponses, HttpServletResponse.SC_FOUND );
         }
         finally
         {
@@ -729,7 +813,8 @@ public abstract class HttpWagonTestCase
 
         String redirectUrl = protocol + "://localhost:" + realServer.getConnectors()[0].getLocalPort();
 
-        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, repositoryDirectory );
+        RedirectHandler redirectHandler =
+            new RedirectHandler( "Found", HttpServletResponse.SC_SEE_OTHER, redirectUrl, repositoryDirectory );
 
         redirectServer.setHandler( redirectHandler );
 
@@ -776,13 +861,8 @@ public abstract class HttpWagonTestCase
     protected void checkRequestResponseForRedirectPutFromStreamWithFullUrl( PutHandler putHandler,
                                                                             RedirectHandler redirectHandler )
     {
-        assertEquals( "found:" + putHandler.handlerRequestResponses, 1, putHandler.handlerRequestResponses.size() );
-        assertEquals( "found:" + putHandler.handlerRequestResponses, 201,
-                      putHandler.handlerRequestResponses.get( 0 ).responseCode );
-        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 1,
-                      redirectHandler.handlerRequestResponses.size() );
-        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 302,
-                      redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
+        checkHandlerResult( putHandler.handlerRequestResponses, HttpServletResponse.SC_CREATED );
+        checkHandlerResult( redirectHandler.handlerRequestResponses, HttpServletResponse.SC_FOUND );
     }
 
     public void testRedirectPutFromStreamRelativeUrl()
@@ -805,7 +885,8 @@ public abstract class HttpWagonTestCase
         addConnectors( redirectServer );
 
         RedirectHandler redirectHandler =
-            new RedirectHandler( "Found", 303, "/redirectRequest/foo", repositoryDirectory );
+            new RedirectHandler( "Found", HttpServletResponse.SC_SEE_OTHER, "/redirectRequest/foo",
+                                 repositoryDirectory );
 
         redirectServer.setHandler( redirectHandler );
 
@@ -853,15 +934,27 @@ public abstract class HttpWagonTestCase
     protected void checkRequestResponseForRedirectPutFromStreamWithRelativeUrl( PutHandler putHandler,
                                                                                 RedirectHandler redirectHandler )
     {
-        assertEquals( "found:" + putHandler.handlerRequestResponses, 0, putHandler.handlerRequestResponses.size() );
+        checkHandlerResult( putHandler.handlerRequestResponses );
+        checkHandlerResult( redirectHandler.handlerRequestResponses, HttpServletResponse.SC_FOUND,
+                            HttpServletResponse.SC_CREATED );
+    }
 
-        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 2,
-                      redirectHandler.handlerRequestResponses.size() );
-        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 302,
-                      redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
-        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 201,
-                      redirectHandler.handlerRequestResponses.get( 1 ).responseCode );
+    protected void checkHandlerResult( List<HandlerRequestResponse> handlerRequestResponses,
+                                       int... expectedResponseCodes )
+    {
+        boolean success = true;
+        if ( handlerRequestResponses.size() == expectedResponseCodes.length )
+        {
+            for ( int i = 0; i < expectedResponseCodes.length; i++ )
+            {
+                success &= ( expectedResponseCodes[i] == handlerRequestResponses.get( i ).responseCode );
+            }
+        }
 
+        if ( !success )
+        {
+            fail( "expected " + expectedResponseCodes + ", got " + handlerRequestResponses );
+        }
     }
 
     public void testRedirectPutFileWithFullUrl()
@@ -900,7 +993,8 @@ public abstract class HttpWagonTestCase
 
         String redirectUrl = protocol + "://localhost:" + realServer.getConnectors()[0].getLocalPort();
 
-        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, repositoryDirectory );
+        RedirectHandler redirectHandler =
+            new RedirectHandler( "Found", HttpServletResponse.SC_SEE_OTHER, redirectUrl, repositoryDirectory );
 
         redirectServer.setHandler( redirectHandler );
 
@@ -961,7 +1055,8 @@ public abstract class HttpWagonTestCase
         addConnectors( redirectServer );
 
         RedirectHandler redirectHandler =
-            new RedirectHandler( "Found", 303, "/redirectRequest/foo", repositoryDirectory );
+            new RedirectHandler( "Found", HttpServletResponse.SC_SEE_OTHER, "/redirectRequest/foo",
+                                 repositoryDirectory );
 
         redirectServer.setHandler( redirectHandler );
 
@@ -1002,6 +1097,10 @@ public abstract class HttpWagonTestCase
     }
 
 
+    /**
+     * 
+     */
+    @SuppressWarnings( "checkstyle:visibilitymodifier" )
     public static class RedirectHandler
         extends AbstractHandler
     {
@@ -1049,7 +1148,9 @@ public abstract class HttpWagonTestCase
         // what an UGLY hack!
         // but apparently jetty needs some time to free up resources
         // <5s: broken test :(
+        // CHECKSTYLE_OFF: MagicNumber
         Thread.sleep( 5001L );
+        // CHECKSTYLE_ON: MagicNumber
 
         Server proxyServer = new Server( 0 );
 
@@ -1099,6 +1200,64 @@ public abstract class HttpWagonTestCase
         }
     }
 
+    private void runTestProxiedRequestWithProvider( ProxyInfoProvider proxyInfoProvider, TestHeaderHandler handler )
+        throws Exception
+    {
+        // what an UGLY hack!
+        // but apparently jetty needs some time to free up resources
+        // <5s: broken test :(
+        // CHECKSTYLE_OFF: MagicNumber
+        Thread.sleep( 5001L );
+        // CHECKSTYLE_ON: MagicNumber
+
+        Server proxyServer = new Server( 0 );
+
+        proxyServer.setHandler( handler );
+
+        proxyServer.start();
+
+        proxyInfoProvider.getProxyInfo( null ).setPort( proxyServer.getConnectors()[0].getLocalPort() );
+
+        System.out.println( "start proxy on host/port " + proxyInfoProvider.getProxyInfo( null ).getHost() + "/"
+                                + proxyInfoProvider.getProxyInfo( null ).getPort() + " with non proxyHosts "
+                                + proxyInfoProvider.getProxyInfo( null ).getNonProxyHosts() );
+
+        while ( !proxyServer.isRunning() || !proxyServer.isStarted() )
+        {
+            Thread.sleep( 10 );
+        }
+
+        try
+        {
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+
+            Repository testRepository = new Repository( "id", "http://www.example.com/" );
+
+            String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
+            File sourceFile = new File( localRepositoryPath, "test-proxied-resource" );
+            FileUtils.fileWrite( sourceFile.getAbsolutePath(), "content" );
+
+            wagon.connect( testRepository, proxyInfoProvider );
+
+            try
+            {
+                wagon.getToStream( "test-proxied-resource", new ByteArrayOutputStream() );
+
+                assertTrue( handler.headers.containsKey( "Proxy-Connection" ) );
+            }
+            finally
+            {
+                System.setProperty( "http.proxyHost", "" );
+                System.setProperty( "http.proxyPort", "" );
+                wagon.disconnect();
+            }
+        }
+        finally
+        {
+            proxyServer.stop();
+        }
+    }
+
     private ProxyInfo createProxyInfo()
     {
         ProxyInfo proxyInfo = new ProxyInfo();
@@ -1439,7 +1598,7 @@ public abstract class HttpWagonTestCase
                     else
                     {
                         called.set( true );
-                        response.setStatus( 429 );
+                        response.setStatus( SC_TOO_MANY_REQUESTS );
                         ( (Request) request ).setHandled( true );
                     }
                 }
@@ -1727,14 +1886,14 @@ public abstract class HttpWagonTestCase
         {
             assertEquals( "not 1 security handler use " + sh.handlerRequestResponses, 1,
                           sh.handlerRequestResponses.size() );
-            assertEquals( 200, sh.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( HttpServletResponse.SC_OK, sh.handlerRequestResponses.get( 0 ).responseCode );
         }
         else
         {
             assertEquals( "not 2 security handler use " + sh.handlerRequestResponses, 2,
                           sh.handlerRequestResponses.size() );
-            assertEquals( 401, sh.handlerRequestResponses.get( 0 ).responseCode );
-            assertEquals( 200, sh.handlerRequestResponses.get( 1 ).responseCode );
+            assertEquals( HttpServletResponse.SC_UNAUTHORIZED, sh.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( HttpServletResponse.SC_OK, sh.handlerRequestResponses.get( 1 ).responseCode );
 
         }
     }
@@ -1789,6 +1948,10 @@ public abstract class HttpWagonTestCase
         }
     }
 
+    /**
+     * 
+     */
+    @SuppressWarnings( "checkstyle:visibilitymodifier" )
     public static class PutHandler
         extends AbstractHandler
     {
@@ -1808,15 +1971,15 @@ public abstract class HttpWagonTestCase
         public void handle( String target, HttpServletRequest request, HttpServletResponse response, int dispatch )
             throws IOException, ServletException
         {
-            Request base_request =
+            Request baseRequest =
                 request instanceof Request ? (Request) request : HttpConnection.getCurrentConnection().getRequest();
 
-            if ( base_request.isHandled() || !"PUT".equals( base_request.getMethod() ) )
+            if ( baseRequest.isHandled() || !"PUT".equals( baseRequest.getMethod() ) )
             {
                 return;
             }
 
-            base_request.setHandled( true );
+            baseRequest.setHandled( true );
 
             File file = new File( resourceBase, URLDecoder.decode( request.getPathInfo() ) );
             file.getParentFile().mkdirs();
@@ -1861,19 +2024,25 @@ public abstract class HttpWagonTestCase
             if ( request.getHeader( "Proxy-Authorization" ) == null )
             {
                 handlerRequestResponses.add(
-                    new HandlerRequestResponse( request.getMethod(), 407, request.getRequestURI() ) );
-                response.setStatus( 407 );
+                    new HandlerRequestResponse( request.getMethod(),
+                                                HttpServletResponse.SC_PROXY_AUTHENTICATION_REQUIRED,
+                                                request.getRequestURI() ) );
+                response.setStatus( HttpServletResponse.SC_PROXY_AUTHENTICATION_REQUIRED );
                 response.addHeader( "Proxy-Authenticate", "Basic realm=\"Squid proxy-caching web server\"" );
 
                 ( (Request) request ).setHandled( true );
                 return;
             }
             handlerRequestResponses.add(
-                new HandlerRequestResponse( request.getMethod(), 200, request.getRequestURI() ) );
+                new HandlerRequestResponse( request.getMethod(), HttpServletResponse.SC_OK, request.getRequestURI() ) );
             super.handle( target, request, response, dispatch );
         }
     }
 
+    /**
+     * 
+     */
+    @SuppressWarnings( "checkstyle:visibilitymodifier" )
     private static class TestHeaderHandler
         extends AbstractHandler
     {
@@ -1892,7 +2061,20 @@ public abstract class HttpWagonTestCase
             for ( Enumeration<String> e = request.getHeaderNames(); e.hasMoreElements(); )
             {
                 String name = e.nextElement();
-                headers.put( name, request.getHeader( name ) );
+                Enumeration headerValues = request.getHeaders( name );
+                // as per HTTP spec http://www.w3.org/Protocols/rfc2616/rfc2616-sec4.html
+                // multiple values for the same header key are concatenated separated by comma
+                // otherwise we wouldn't notice headers with same key added multiple times
+                StringBuffer combinedHeaderValue = new StringBuffer();
+                for ( int i = 0; headerValues.hasMoreElements(); i++ )
+                {
+                    if ( i > 0 )
+                    {
+                        combinedHeaderValue.append( "," );
+                    }
+                    combinedHeaderValue.append( headerValues.nextElement() );
+                }
+                headers.put( name, combinedHeaderValue.toString() );
             }
 
             response.setContentType( "text/plain" );
@@ -1928,6 +2110,10 @@ public abstract class HttpWagonTestCase
         return sh;
     }
 
+    /**
+     * 
+     */
+    @SuppressWarnings( "checkstyle:visibilitymodifier" )
     public static class TestSecurityHandler
         extends SecurityHandler
     {
@@ -1947,6 +2133,10 @@ public abstract class HttpWagonTestCase
 
     }
 
+    /**
+     * 
+     */
+    @SuppressWarnings( "checkstyle:visibilitymodifier" )
     public static class HandlerRequestResponse
     {
         public String method;
