@@ -19,25 +19,6 @@ package org.apache.maven.wagon.providers.http;
  * under the License.
  */
 
-import java.io.ByteArrayInputStream;
-import java.io.Closeable;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.nio.ByteBuffer;
-import java.text.SimpleDateFormat;
-import java.util.Date;
-import java.util.Locale;
-import java.util.Map;
-import java.util.Properties;
-import java.util.TimeZone;
-import java.util.concurrent.TimeUnit;
-
-import javax.net.ssl.HttpsURLConnection;
-import javax.net.ssl.SSLContext;
-
 import org.apache.http.Header;
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpException;
@@ -63,6 +44,7 @@ import org.apache.http.client.protocol.HttpClientContext;
 import org.apache.http.client.utils.DateUtils;
 import org.apache.http.config.Registry;
 import org.apache.http.config.RegistryBuilder;
+import org.apache.http.conn.HttpClientConnectionManager;
 import org.apache.http.conn.socket.ConnectionSocketFactory;
 import org.apache.http.conn.socket.PlainConnectionSocketFactory;
 import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
@@ -123,7 +105,7 @@ public abstract class AbstractHttpClientWagon
         extends AbstractHttpEntity
     {
 
-        private final static int BUFFER_SIZE = 2048;
+        private static final int BUFFER_SIZE = 2048;
 
         private final Resource resource;
 
@@ -243,28 +225,28 @@ public abstract class AbstractHttpClientWagon
      * use http(s) connection pool mechanism.
      * <b>enabled by default</b>
      */
-    private final static boolean PERSISTENT_POOL =
+    private static boolean persistentPool =
         Boolean.valueOf( System.getProperty( "maven.wagon.http.pool", "true" ) );
 
     /**
      * skip failure on certificate validity checks.
      * <b>disabled by default</b>
      */
-    private final static boolean SSL_INSECURE =
+    private static final boolean SSL_INSECURE =
         Boolean.valueOf( System.getProperty( "maven.wagon.http.ssl.insecure", "false" ) );
 
     /**
      * if using sslInsecure, certificate date issues will be ignored
      * <b>disabled by default</b>
      */
-    private final static boolean IGNORE_SSL_VALIDITY_DATES =
+    private static final boolean IGNORE_SSL_VALIDITY_DATES =
         Boolean.valueOf( System.getProperty( "maven.wagon.http.ssl.ignore.validity.dates", "false" ) );
 
     /**
-     * If enabled, ssl hostname verifier does not check hostname. Disable this will use a browser compat hostname verifier
-     * <b>disabled by default</b>
+     * If enabled, ssl hostname verifier does not check hostname. Disable this will use a browser compat hostname
+     * verifier <b>disabled by default</b>
      */
-    private final static boolean SSL_ALLOW_ALL =
+    private static final boolean SSL_ALLOW_ALL =
         Boolean.valueOf( System.getProperty( "maven.wagon.http.ssl.allowall", "false" ) );
 
 
@@ -272,20 +254,20 @@ public abstract class AbstractHttpClientWagon
      * Maximum concurrent connections per distinct route.
      * <b>20 by default</b>
      */
-    private final static int MAX_CONN_PER_ROUTE =
+    private static final int MAX_CONN_PER_ROUTE =
         Integer.parseInt( System.getProperty( "maven.wagon.httpconnectionManager.maxPerRoute", "20" ) );
 
     /**
      * Maximum concurrent connections in total.
      * <b>40 by default</b>
      */
-    private final static int MAX_CONN_TOTAL =
+    private static final int MAX_CONN_TOTAL =
         Integer.parseInt( System.getProperty( "maven.wagon.httpconnectionManager.maxTotal", "40" ) );
 
     /**
      * Internal connection manager
      */
-    private static final PoolingHttpClientConnectionManager CONN_MAN = createConnManager();
+    private static HttpClientConnectionManager httpClientConnectionManager = createConnManager();
 
 
     /**
@@ -313,7 +295,7 @@ public abstract class AbstractHttpClientWagon
      *
      * @since 2.7
      */
-    private final static int maxBackoffWaitSeconds =
+    private static final int MAX_BACKOFF_WAIT_SECONDS =
         Integer.parseInt( System.getProperty( "maven.wagon.httpconnectionManager.maxBackoffSeconds", "180" ) );
 
 
@@ -330,7 +312,7 @@ public abstract class AbstractHttpClientWagon
         return nextWait;
     }
 
-
+    @SuppressWarnings( "checkstyle:linelength" )
     private static PoolingHttpClientConnectionManager createConnManager()
     {
 
@@ -370,7 +352,7 @@ public abstract class AbstractHttpClientWagon
             "https", sslConnectionSocketFactory ).build();
 
         PoolingHttpClientConnectionManager connManager = new PoolingHttpClientConnectionManager( registry );
-        if ( PERSISTENT_POOL )
+        if ( persistentPool )
         {
             connManager.setDefaultMaxPerRoute( MAX_CONN_PER_ROUTE );
             connManager.setMaxTotal( MAX_CONN_TOTAL );
@@ -382,45 +364,17 @@ public abstract class AbstractHttpClientWagon
         return connManager;
     }
 
-    private static CloseableHttpClient CLIENT = createClient();
+    private static final CloseableHttpClient CLIENT = createClient();
 
     private static CloseableHttpClient createClient()
     {
         return HttpClientBuilder.create() //
             .useSystemProperties() //
             .disableConnectionState() //
-            .setConnectionManager( CONN_MAN ) //
+            .setConnectionManager( httpClientConnectionManager ) //
             .build();
     }
 
-    private static String DEFAULT_USER_AGENT = getDefaultUserAgent();
-
-    private static String getDefaultUserAgent()
-    {
-        Properties props = new Properties();
-
-        InputStream is = AbstractHttpClientWagon.class.getResourceAsStream(
-            "/META-INF/maven/org.apache.maven.wagon/wagon-http/pom.properties" );
-        if ( is != null )
-        {
-            try
-            {
-                props.load( is );
-            }
-            catch ( IOException ignore )
-            {
-            }
-            finally
-            {
-                IOUtil.close( is );
-            }
-        }
-
-        String ver = props.getProperty( "version", "unknown-version" );
-        return "Apache-Maven-Wagon/" + ver + " (Java " + System.getProperty( "java.version" ) + "; ";
-    }
-
-
     private CredentialsProvider credentialsProvider;
 
     private AuthCache authCache;
@@ -440,6 +394,18 @@ public abstract class AbstractHttpClientWagon
      */
     private HttpConfiguration httpConfiguration;
 
+    /**
+     * Basic auth scope overrides
+     * @since 2.8
+     */
+    private BasicAuthScope basicAuth;
+
+    /**
+     * Proxy basic auth scope overrides
+     * @since 2.8
+     */
+    private BasicAuthScope proxyAuth;
+
     public void openConnectionInternal()
     {
         repository.setUrl( getURL( repository ) );
@@ -461,9 +427,9 @@ public abstract class AbstractHttpClientWagon
                 Credentials creds = new UsernamePasswordCredentials( username, password );
 
                 String host = getRepository().getHost();
-                int port = getRepository().getPort() > -1 ? getRepository().getPort() : AuthScope.ANY_PORT;
+                int port = getRepository().getPort();
 
-                credentialsProvider.setCredentials( new AuthScope( host, port ), creds );
+                credentialsProvider.setCredentials( getBasicAuthScope().getScope( host, port ), creds );
             }
         }
 
@@ -489,9 +455,9 @@ public abstract class AbstractHttpClientWagon
                         creds = new UsernamePasswordCredentials( proxyUsername, proxyPassword );
                     }
 
-                    int port = proxyInfo.getPort() > -1 ? proxyInfo.getPort() : AuthScope.ANY_PORT;
+                    int port = proxyInfo.getPort();
 
-                    AuthScope authScope = new AuthScope( proxyHost, port );
+                    AuthScope authScope = getProxyBasicAuthScope().getScope( proxyHost, port );
                     credentialsProvider.setCredentials( authScope, creds );
                 }
             }
@@ -500,12 +466,23 @@ public abstract class AbstractHttpClientWagon
 
     public void closeConnection()
     {
-        if ( !PERSISTENT_POOL )
+        if ( !persistentPool )
         {
-            CONN_MAN.closeIdleConnections( 0, TimeUnit.MILLISECONDS );
+            httpClientConnectionManager.closeIdleConnections( 0, TimeUnit.MILLISECONDS );
         }
     }
 
+    public static void setPersistentPool( boolean persistentPool )
+    {
+        persistentPool = persistentPool;
+    }
+
+    public static void setPoolingHttpClientConnectionManager(
+        PoolingHttpClientConnectionManager poolingHttpClientConnectionManager )
+    {
+        httpClientConnectionManager = poolingHttpClientConnectionManager;
+    }
+
     public void put( File source, String resourceName )
         throws TransferFailedException, ResourceDoesNotExistException, AuthorizationException
     {
@@ -554,7 +531,7 @@ public abstract class AbstractHttpClientWagon
      */
     private String buildUrl( Resource resource )
     {
-    	return EncodingUtil.encodeURLToString( getRepository().getUrl(), resource.getName() );
+        return EncodingUtil.encodeURLToString( getRepository().getUrl(), resource.getName() );
     }
 
 
@@ -588,7 +565,7 @@ public abstract class AbstractHttpClientWagon
 
         Repository repo = getRepository();
         HttpHost targetHost = new HttpHost( repo.getHost(), repo.getPort(), repo.getProtocol() );
-        AuthScope targetScope = new AuthScope( targetHost );
+        AuthScope targetScope = getBasicAuthScope().getScope( targetHost );
 
         if ( credentialsProvider.getCredentials( targetScope ) != null )
         {
@@ -600,6 +577,7 @@ public abstract class AbstractHttpClientWagon
             }
             catch ( MalformedChallengeException ignore )
             {
+                // ignore
             }
         }
 
@@ -627,7 +605,8 @@ public abstract class AbstractHttpClientWagon
                     case HttpStatus.SC_ACCEPTED: // 202
                     case HttpStatus.SC_NO_CONTENT:  // 204
                         break;
-                    // handle all redirect even if http specs says " the user agent MUST NOT automatically redirect the request unless it can be confirmed by the user"
+                    // handle all redirect even if http specs says " the user agent MUST NOT automatically redirect
+                    // the request unless it can be confirmed by the user"
                     case HttpStatus.SC_MOVED_PERMANENTLY: // 301
                     case HttpStatus.SC_MOVED_TEMPORARILY: // 302
                     case HttpStatus.SC_SEE_OTHER: // 303
@@ -645,12 +624,10 @@ public abstract class AbstractHttpClientWagon
                         break;
                     //add more entries here
                     default:
-                    {
                         TransferFailedException e = new TransferFailedException(
                             "Failed to transfer file: " + url + ". Return code is: " + statusCode + reasonPhrase );
                         fireTransferError( resource, e, TransferEvent.REQUEST_PUT );
                         throw e;
-                    }
                 }
 
                 firePutCompleted( resource, source );
@@ -810,7 +787,7 @@ public abstract class AbstractHttpClientWagon
         if ( config != null && config.isUsePreemptive() )
         {
             HttpHost targetHost = new HttpHost( repo.getHost(), repo.getPort(), repo.getProtocol() );
-            AuthScope targetScope = new AuthScope( targetHost );
+            AuthScope targetScope = getBasicAuthScope().getScope( targetHost );
 
             if ( credentialsProvider.getCredentials( targetScope ) != null )
             {
@@ -825,7 +802,7 @@ public abstract class AbstractHttpClientWagon
             if ( proxyInfo.getHost() != null )
             {
                 HttpHost proxyHost = new HttpHost( proxyInfo.getHost(), proxyInfo.getPort() );
-                AuthScope proxyScope = new AuthScope( proxyHost );
+                AuthScope proxyScope = getProxyBasicAuthScope().getScope( proxyHost );
 
                 String proxyUsername = proxyInfo.getUserName();
                 String proxyPassword = proxyInfo.getPassword();
@@ -867,14 +844,13 @@ public abstract class AbstractHttpClientWagon
             method.addHeader( "Pragma", "no-cache" );
             method.addHeader( "Expires", "0" );
             method.addHeader( "Accept-Encoding", "gzip" );
-            method.addHeader( "User-Agent", DEFAULT_USER_AGENT );
         }
 
         if ( httpHeaders != null )
         {
             for ( Map.Entry<Object, Object> entry : httpHeaders.entrySet() )
             {
-                method.addHeader( (String) entry.getKey(), (String) entry.getValue() );
+                method.setHeader( (String) entry.getKey(), (String) entry.getValue() );
             }
         }
 
@@ -883,7 +859,7 @@ public abstract class AbstractHttpClientWagon
         {
             for ( Header header : headers )
             {
-                method.addHeader( header );
+                method.setHeader( header );
             }
         }
     }
@@ -930,6 +906,54 @@ public abstract class AbstractHttpClientWagon
         this.httpConfiguration = httpConfiguration;
     }
 
+    /**
+     * Get the override values for standard HttpClient AuthScope
+     *
+     * @return the basicAuth
+     */
+    public BasicAuthScope getBasicAuthScope()
+    {
+        if ( basicAuth == null )
+        {
+            basicAuth = new BasicAuthScope();
+        }
+        return basicAuth;
+    }
+
+    /**
+     * Set the override values for standard HttpClient AuthScope
+     *
+     * @param basicAuth the AuthScope to set
+     */
+    public void setBasicAuthScope( BasicAuthScope basicAuth )
+    {
+        this.basicAuth = basicAuth;
+    }
+
+    /**
+     * Get the override values for proxy HttpClient AuthScope
+     *
+     * @return the proxyAuth
+     */
+    public BasicAuthScope getProxyBasicAuthScope()
+    {
+        if ( proxyAuth == null )
+        {
+            proxyAuth = new BasicAuthScope();
+        }
+        return proxyAuth;
+    }
+
+    /**
+     * Set the override values for proxy HttpClient AuthScope
+     *
+     * @param proxyAuth the AuthScope to set
+     */
+    public void setProxyBasicAuthScope( BasicAuthScope proxyAuth )
+    {
+        this.proxyAuth = proxyAuth;
+    }
+
     public void fillInputData( InputData inputData )
         throws TransferFailedException, ResourceDoesNotExistException, AuthorizationException
     {
@@ -993,13 +1017,11 @@ public abstract class AbstractHttpClientWagon
 
                 // add more entries here
                 default:
-                {
                     cleanupGetTransfer( resource );
                     TransferFailedException e = new TransferFailedException(
                         "Failed to transfer file: " + url + ". Return code is: " + statusCode + " " + reasonPhrase );
                     fireTransferError( resource, e, TransferEvent.REQUEST_GET );
                     throw e;
-                }
             }
 
             Header contentLengthHeader = response.getFirstHeader( "Content-Length" );
@@ -1026,8 +1048,8 @@ public abstract class AbstractHttpClientWagon
                 if ( lastModified != null )
                 {
                     resource.setLastModified( lastModified.getTime() );
-                    fireTransferDebug( "last-modified = " + lastModifiedHeader.getValue() +
-                                           " (" + lastModified.getTime() + ")" );
+                    fireTransferDebug( "last-modified = " + lastModifiedHeader.getValue() + " ("
+                        + lastModified.getTime() + ")" );
                 }
             }
 
@@ -1068,6 +1090,7 @@ public abstract class AbstractHttpClientWagon
             }
             catch ( IOException ignore )
             {
+                // ignore
             }
 
         }
@@ -1118,6 +1141,6 @@ public abstract class AbstractHttpClientWagon
 
     public static int getMaxBackoffWaitSeconds()
     {
-        return maxBackoffWaitSeconds;
+        return MAX_BACKOFF_WAIT_SECONDS;
     }
 }
\ No newline at end of file
