@@ -54,6 +54,7 @@ import org.apache.maven.wagon.proxy.ProxyInfo;
 import org.apache.maven.wagon.repository.Repository;
 import org.apache.maven.wagon.resource.Resource;
 import org.apache.maven.wagon.shared.http.EncodingUtil;
+import org.codehaus.plexus.util.IOUtil;
 
 import java.io.ByteArrayInputStream;
 import java.io.File;
@@ -376,6 +377,7 @@ public abstract class AbstractHttpClientWagon
             fireTransferDebug( url + " - Status code: " + statusCode );
 
             // Check that we didn't run out of retries.
+            // CHECKSTYLE_OFF: AvoidNestedBlocks
             switch ( statusCode )
             {
                 // Success Codes
@@ -385,7 +387,8 @@ public abstract class AbstractHttpClientWagon
                 case HttpStatus.SC_NO_CONTENT:  // 204
                     break;
 
-                // handle all redirect even if http specs says " the user agent MUST NOT automatically redirect the request unless it can be confirmed by the user"
+                // handle all redirect even if http specs says
+                // " the user agent MUST NOT automatically redirect the request unless it can be confirmed by the user"
                 case HttpStatus.SC_MOVED_PERMANENTLY: // 301
                 case HttpStatus.SC_MOVED_TEMPORARILY: // 302
                 case HttpStatus.SC_SEE_OTHER: // 303
@@ -417,6 +420,7 @@ public abstract class AbstractHttpClientWagon
                     throw e;
                 }
             }
+            // CHECKSTYLE_ON: AvoidNestedBlocks
 
             firePutCompleted( resource, source );
         }
@@ -540,13 +544,21 @@ public abstract class AbstractHttpClientWagon
             method.addRequestHeader( "Pragma", "no-cache" );
             method.addRequestHeader( "Expires", "0" );
             method.addRequestHeader( "Accept-Encoding", "gzip" );
+            method.addRequestHeader( "User-Agent", DEFAULT_USER_AGENT );
         }
 
         if ( httpHeaders != null )
         {
             for ( Object header : httpHeaders.keySet() )
             {
-                method.addRequestHeader( (String) header, httpHeaders.getProperty( (String) header ) );
+                if ( "User-Agent".equals( header ) )
+                {
+                    method.setRequestHeader( (String) header, httpHeaders.getProperty( (String) header ) );
+                }
+                else
+                {
+                    method.addRequestHeader( (String) header, httpHeaders.getProperty( (String) header ) );
+                }
             }
         }
 
@@ -559,6 +571,34 @@ public abstract class AbstractHttpClientWagon
             }
         }
     }
+    
+    private static final String DEFAULT_USER_AGENT = getDefaultUserAgent();
+
+    private static String getDefaultUserAgent()
+    {
+        Properties props = new Properties();
+
+        InputStream is = AbstractHttpClientWagon.class.getResourceAsStream(
+            "/META-INF/maven/org.apache.maven.wagon/wagon-webdav-jackrabbit/pom.properties" );
+        if ( is != null )
+        {
+            try
+            {
+                props.load( is );
+            }
+            catch ( IOException ignore )
+            {
+                // ignore
+            }
+            finally
+            {
+                IOUtil.close( is );
+            }
+        }
+
+        String ver = props.getProperty( "version", "unknown-version" );
+        return "Apache-Maven-Wagon/" + ver + " (Java " + System.getProperty( "java.version" ) + "; ";
+    }
 
     /**
      * getUrl
@@ -642,6 +682,7 @@ public abstract class AbstractHttpClientWagon
 
         // TODO [BP]: according to httpclient docs, really should swallow the output on error. verify if that is
         // required
+        // CHECKSTYLE_OFF: AvoidNestedBlocks
         switch ( statusCode )
         {
             case HttpStatus.SC_OK:
@@ -683,6 +724,7 @@ public abstract class AbstractHttpClientWagon
                 throw e;
             }
         }
+        // CHECKSTYLE_ON: AvoidNestedBlocks
 
         InputStream is = null;
 
